#pragma once

#include "IHair.h"

namespace Ephere { namespace Plugins { namespace Ornatrix
{

HostVector3& IHair::GetVertex( unsigned vertexIndex )
{
	return const_cast<HostVector3&>( const_cast<const IHair&>( *this ).GetVertex( vertexIndex ) );
}


HostVector3* IHair::GetStrandPoints( unsigned rootIndex )
{
	return const_cast<HostVector3*>( const_cast<const IHair&>( *this ).GetStrandPoints( rootIndex ) );
}

void IHair::SetStrandPointInObjectCoordinates( unsigned strandIndex, unsigned pointIndex, const HostVector3& value )
{

	if ( UsesPerStrandTransformations() )
	{
		SetStrandPoint( strandIndex, pointIndex, HostTransform( HostInverse( GetStrandTransform( strandIndex ) ), value ) );
	}
	else
	{
		SetStrandPoint( strandIndex, pointIndex, value );
	}
}

void IHair::GetVertexStrandAndPointIndices( unsigned vertexIndex, unsigned& strandIndexResult, unsigned& pointIndexResult ) const
{
	for ( auto strandIndex = 0u, strandCount = GetStrandCount(); strandIndex < strandCount; ++strandIndex )
	{
		auto firstVertex = GetFirstVertexIndex( strandIndex );
		auto vertexCount = GetStrandPointCount( strandIndex );
		auto pointIndex = vertexIndex - firstVertex;
		if ( pointIndex < vertexCount )
		{
			strandIndexResult = strandIndex;
			pointIndexResult = pointIndex;
			return;
		}
	}

	//CHECK_ARGUMENT_RANGE( strandIndex < GetStrandCount() );
}

HostVector3 IHair::GetVertexInObjectCoordinates( unsigned vertexIndex ) const
{
	HostVector3 result;
	if ( UsesPerStrandTransformations() )
	{
		unsigned strandIndex, pointIndex;
		GetVertexStrandAndPointIndices( vertexIndex, strandIndex, pointIndex );
		result = GetStrandPointInObjectCoordinates( strandIndex, pointIndex );
	}
	else
	{
		result = GetVertex( vertexIndex );
	}
	return result;
}

void IHair::SetVertexInObjectCoordinates( unsigned vertexIndex, const HostVector3& value )
{
	if ( UsesPerStrandTransformations() )
	{
		unsigned strandIndex, pointIndex;
		GetVertexStrandAndPointIndices( vertexIndex, strandIndex, pointIndex );
		SetStrandPointInObjectCoordinates( strandIndex, pointIndex, value );
	}
	else
	{
		SetVertex( vertexIndex, value );
	}
}

IGuides* IHair::AsGuides()
{
	return const_cast<IGuides*>( const_cast<const IHair&>( *this ).AsGuides() );
}

void IHair::CopyFlags( const IHair& source )
{
	UsesStrandTopology( source.UsesStrandTopology() );
	UsesPerStrandTransformations( source.UsesPerStrandTransformations() );
	KeepsSurfaceDependency( source.KeepsSurfaceDependency() );
	KeepsGuideDependency( source.KeepsGuideDependency() );
	UsesTopologyOrientation( source.UsesTopologyOrientation() );
	UsesGlobalPerStrandPointCount( source.UsesGlobalPerStrandPointCount() );
	SetUsingPerStrandRotationAngles( source.IsUsingPerStrandRotationAngles() );
	UsesVertexLookupTable( source.UsesVertexLookupTable() );
}

void IHair::ClearFlags()
{
	UsesStrandTopology( false );
	UsesPerStrandTransformations( false );
	KeepsSurfaceDependency( false );
	KeepsGuideDependency( false );
	UsesTopologyOrientation( false );
	UsesGlobalPerStrandPointCount( false );
	SetUsingPerStrandRotationAngles( false );
	UsesVertexLookupTable( false );
}

} } } // Ephere::Plugins::Ornatrix