#pragma once

#include "Ephere/Plugins/AutodeskMaxTypes.h"
#include "Ephere/Plugins/AutodeskMayaTypes.h"
#include "IFunction.h"

namespace Ephere { namespace Plugins { namespace Ornatrix
{

//! Interface to properties that specify how to render an IHair interface
class IHairRenderingProperties
{
public:

	// Properties:

	//! Gets whether this class needs to be used or ignored
	virtual bool IsUsed() const = 0;

	//! Sets whether this class needs to be used or ignored
	virtual void SetUsed( bool value ) = 0;

	//! Gets to which strand group these properties need to be applied. Different properties can be applied to different groups.
	virtual unsigned short GetStrandGroupIndex() const = 0;

	//! Sets to which strand group these properties need to be applied
	virtual void SetStrandGroupIndex( unsigned short value ) = 0;

	// Strand radius:

	//! Gets global radius to be used (or pre-multiplied) for all strands
	virtual float GetGlobalStrandRadius() const = 0;

	//! Sets global radius to be used (or pre-multiplied) for all strands
	virtual void SetGlobalStrandRadius( float value ) = 0;

	//! Gets a thickness map to be used to determine strand radius on per-strand basis, or 0 if none
	virtual HostTextureMap* GetRadiusMap() const = 0;

	//! Sets a thickness map to be used to determine strand radius on per-strand basis, or 0 if none
	virtual void SetRadiusMap( HostTextureMap* value ) = 0;

	//! Gets strand data channel to be used to determine strand thickness, or unsigned( -1 ) if none
	virtual unsigned GetRadiusStrandDataChannel() const = 0;

	//! Sets strand data channel to be used to determine strand thickness, or unsigned( -1 ) if none
	virtual void SetRadiusStrandDataChannel( unsigned value ) = 0;

	//! Gets a curve to determine global strand thickness on points along the strand, or 0 if none
	virtual const IFunction1* GetRadiusCurve() const = 0;

	//! Sets a curve to determine global strand thickness on points along the strand, or 0 if none
	virtual void SetRadiusCurve( const IFunction1* curve ) = 0;

	// Self shadowing:

	//! Gets whether self-shadowing is used
	virtual bool IsUsingSelfShadowing() const = 0;

	//! Sets whether self-shadowing is used
	virtual void SetUsingSelfShadowing( bool value ) = 0;

	//! Gets self-shadowing coefficient
	virtual float GetSelfShadowingCoefficient() const = 0;

	//! Sets self-shadowing coefficient
	virtual void SetSelfShadowingCoefficient( float value ) = 0;

};

} } } // Ephere::Plugins::Ornatrix